@@ -16,16 +16,21 @@ from kombu.common import Broadcast
 from kombu.pools import ProducerPool
 from kombu.utils import cached_property, uuid
 from kombu.utils.encoding import safe_repr
+from kombu.utils.functional import maybe_list
 
 from celery import signals
+from celery.five import items, string_t
 from celery.utils.text import indent as textindent
 
 from . import app_or_default
 from . import routes as _routes
 
+__all__ = ['AMQP', 'Queues', 'TaskProducer', 'TaskConsumer']
+
 #: Human readable queue declaration.
 QUEUE_FORMAT = """
-.> %(name)s exchange:%(exchange)s(%(exchange_type)s) binding:%(routing_key)s
+.> {0.name:<16} exchange={0.exchange.name}({0.exchange.type}) \
+key={0.routing_key}
 """
 
 
@@ -46,15 +51,16 @@ class Queues(dict):
     _consume_from = None
 
     def __init__(self, queues=None, default_exchange=None,
-                 create_missing=True, ha_policy=None):
+                 create_missing=True, ha_policy=None, autoexchange=None):
         dict.__init__(self)
         self.aliases = WeakValueDictionary()
         self.default_exchange = default_exchange
         self.create_missing = create_missing
         self.ha_policy = ha_policy
+        self.autoexchange = Exchange if autoexchange is None else autoexchange
         if isinstance(queues, (tuple, list)):
             queues = dict((q.name, q) for q in queues)
-        for name, q in (queues or {}).iteritems():
+        for name, q in items(queues or {}):
             self.add(q) if isinstance(q, Queue) else self.add_compat(name, **q)
 
     def __getitem__(self, name):
@@ -79,11 +85,16 @@ class Queues(dict):
     def add(self, queue, **kwargs):
         """Add new queue.
 
-        :param queue: Name of the queue.
-        :keyword exchange: Name of the exchange.
-        :keyword routing_key: Binding key.
-        :keyword exchange_type: Type of exchange.
-        :keyword \*\*options: Additional declaration options.
+        The first argument can either be a :class:`kombu.Queue` instance,
+        or the name of a queue.  If the former the rest of the keyword
+        arguments are ignored, and options are simply taken from the queue
+        instance.
+
+        :param queue: :class:`kombu.Queue` instance or name of the queue.
+        :keyword exchange: (if named) specifies exchange name.
+        :keyword routing_key: (if named) specifies binding key.
+        :keyword exchange_type: (if named) specifies type of exchange.
+        :keyword \*\*options: (if named) Additional declaration options.
 
         """
         if not isinstance(queue, Queue):
@@ -117,13 +128,8 @@ class Queues(dict):
         active = self.consume_from
         if not active:
             return ''
-        info = [
-            QUEUE_FORMAT.strip() % {
-                'name': (name + ':').ljust(12),
-                'exchange': q.exchange.name,
-                'exchange_type': q.exchange.type,
-                'routing_key': q.routing_key}
-            for name, q in sorted(active.iteritems())]
+        info = [QUEUE_FORMAT.strip().format(q)
+                for _, q in sorted(items(active))]
         if indent_first:
             return textindent('\n'.join(info), indent)
         return info[0] + '\n' + textindent('\n'.join(info[1:]), indent)
@@ -136,23 +142,37 @@ class Queues(dict):
             self._consume_from[q.name] = q
         return q
 
-    def select_subset(self, wanted):
+    def select(self, include):
         """Sets :attr:`consume_from` by selecting a subset of the
         currently defined queues.
 
-        :param wanted: List of wanted queue names.
+        :param include: Names of queues to consume from.
+                        Can be iterable or string.
         """
-        if wanted:
-            self._consume_from = dict((name, self[name]) for name in wanted)
+        if include:
+            self._consume_from = dict((name, self[name])
+                                      for name in maybe_list(include))
+    select_subset = select  # XXX compat
 
-    def select_remove(self, queue):
-        if self._consume_from is None:
-            self.select_subset(k for k in self if k != queue)
-        else:
-            self._consume_from.pop(queue, None)
+    def deselect(self, exclude):
+        """Deselect queues so that they will not be consumed from.
+
+        :param exclude: Names of queues to avoid consuming from.
+                        Can be iterable or string.
+
+        """
+        if exclude:
+            exclude = maybe_list(exclude)
+            if self._consume_from is None:
+                # using selection
+                return self.select(k for k in self if k not in exclude)
+            # using all queues
+            for queue in exclude:
+                self._consume_from.pop(queue, None)
+    select_remove = deselect  # XXX compat
 
     def new_missing(self, name):
-        return Queue(name, Exchange(name), name)
+        return Queue(name, self.autoexchange(name), name)
 
     @property
     def consume_from(self):
@@ -189,14 +209,23 @@ class TaskProducer(Producer):
                      queue=None, now=None, retries=0, chord=None,
                      callbacks=None, errbacks=None, routing_key=None,
                      serializer=None, delivery_mode=None, compression=None,
-                     declare=None, **kwargs):
+                     reply_to=None, time_limit=None, soft_time_limit=None,
+                     declare=None, headers=None,
+                     send_before_publish=signals.before_task_publish.send,
+                     before_receivers=signals.before_task_publish.receivers,
+                     send_after_publish=signals.after_task_publish.send,
+                     after_receivers=signals.after_task_publish.receivers,
+                     send_task_sent=signals.task_sent.send,  # XXX deprecated
+                     sent_receivers=signals.task_sent.receivers,
+                     **kwargs):
         """Send task message."""
+        retry = self.retry if retry is None else retry
 
         qname = queue
         if queue is None and exchange is None:
             queue = self.default_queue
         if queue is not None:
-            if isinstance(queue, basestring):
+            if isinstance(queue, string_t):
                 qname, queue = queue, self.queues[queue]
             else:
                 qname = queue.name
@@ -219,9 +248,13 @@ class TaskProducer(Producer):
         if countdown:  # Convert countdown to ETA.
             now = now or self.app.now()
             eta = now + timedelta(seconds=countdown)
+            if self.utc:
+                eta = eta.replace(tzinfo=self.app.timezone)
         if isinstance(expires, (int, float)):
             now = now or self.app.now()
             expires = now + timedelta(seconds=expires)
+            if self.utc:
+                expires = expires.replace(tzinfo=self.app.timezone)
         eta = eta and eta.isoformat()
         expires = expires and expires.isoformat()
 
@@ -236,21 +269,44 @@ class TaskProducer(Producer):
             'utc': self.utc,
             'callbacks': callbacks,
             'errbacks': errbacks,
+            'timelimit': (time_limit, soft_time_limit),
             'taskset': group_id or taskset_id,
             'chord': chord,
         }
 
+        if before_receivers:
+            send_before_publish(
+                sender=task_name, body=body,
+                exchange=exchange,
+                routing_key=routing_key,
+                declare=declare,
+                headers=headers,
+                properties=kwargs,
+                retry_policy=retry_policy,
+            )
+
         self.publish(
             body,
             exchange=exchange, routing_key=routing_key,
             serializer=serializer or self.serializer,
             compression=compression or self.compression,
+            headers=headers,
             retry=retry, retry_policy=_rp,
+            reply_to=reply_to,
+            correlation_id=task_id,
             delivery_mode=delivery_mode, declare=declare,
             **kwargs
         )
 
-        signals.task_sent.send(sender=task_name, **body)
+        if after_receivers:
+            send_after_publish(sender=task_name, body=body,
+                               exchange=exchange, routing_key=routing_key)
+
+        if sent_receivers:  # XXX deprecated
+            send_task_sent(sender=task_name, task_id=task_id,
+                           task=task_name, args=task_args,
+                           kwargs=task_kwargs, eta=eta,
+                           taskset=group_id or taskset_id)
         if self.send_sent_event:
             evd = event_dispatcher or self.event_dispatcher
             exname = exchange or self.exchange
@@ -307,7 +363,7 @@ class TaskConsumer(Consumer):
             accept = self.app.conf.CELERY_ACCEPT_CONTENT
         super(TaskConsumer, self).__init__(
             channel,
-            queues or self.app.amqp.queues.consume_from.values(),
+            queues or list(self.app.amqp.queues.consume_from.values()),
             accept=accept,
             **kw
         )
@@ -330,13 +386,20 @@ class AMQP(object):
     #: set by the :attr:`producer_pool`.
     _producer_pool = None
 
+    # Exchange class/function used when defining automatic queues.
+    # E.g. you can use ``autoexchange = lambda n: None`` to use the
+    # amqp default exchange, which is a shortcut to bypass routing
+    # and instead send directly to the queue named in the routing key.
+    autoexchange = None
+
     def __init__(self, app):
         self.app = app
 
     def flush_routes(self):
         self._rtable = _routes.prepare(self.app.conf.CELERY_ROUTES)
 
-    def Queues(self, queues, create_missing=None, ha_policy=None):
+    def Queues(self, queues, create_missing=None, ha_policy=None,
+               autoexchange=None):
         """Create new :class:`Queues` instance, using queue defaults
         from the current configuration."""
         conf = self.app.conf
@@ -348,10 +411,15 @@ class AMQP(object):
             queues = (Queue(conf.CELERY_DEFAULT_QUEUE,
                             exchange=self.default_exchange,
                             routing_key=conf.CELERY_DEFAULT_ROUTING_KEY), )
-        return Queues(queues, self.default_exchange, create_missing, ha_policy)
+        autoexchange = (self.autoexchange if autoexchange is None
+                        else autoexchange)
+        return Queues(
+            queues, self.default_exchange, create_missing,
+            ha_policy, autoexchange,
+        )
 
     def Router(self, queues=None, create_missing=None):
-        """Returns the current task router."""
+        """Return the current task router."""
         return _routes.Router(self.routes, queues or self.queues,
                               self.app.either('CELERY_CREATE_MISSING_QUEUES',
                                               create_missing), app=self.app)
@@ -366,7 +434,7 @@ class AMQP(object):
 
     @cached_property
     def TaskProducer(self):
-        """Returns publisher used to send tasks.
+        """Return publisher used to send tasks.
 
         You should use `app.send_task` instead.
 
