@@ -8,32 +8,35 @@
 
 """
 from __future__ import absolute_import
-from __future__ import with_statement
 
 from collections import deque
 
 from celery._state import get_current_worker_task
 from celery.utils import uuid
 
+__all__ = ['shared_task', 'load_shared_tasks']
+
 #: global list of functions defining tasks that should be
 #: added to all apps.
-_shared_tasks = []
+_shared_tasks = set()
 
 
 def shared_task(constructor):
-    """Decorator that specifies that the decorated function is a function
-    that generates a built-in task.
+    """Decorator that specifies a function that generates a built-in task.
 
     The function will then be called for every new app instance created
     (lazily, so more exactly when the task registry for that app is needed).
+
+    The function must take a single ``app`` argument.
     """
-    _shared_tasks.append(constructor)
+    _shared_tasks.add(constructor)
     return constructor
 
 
 def load_shared_tasks(app):
-    """Loads the built-in tasks for an app instance."""
-    for constructor in _shared_tasks:
+    """Create built-in tasks for an app instance."""
+    constructors = set(_shared_tasks)
+    for constructor in constructors:
         constructor(app)
 
 
@@ -42,17 +45,13 @@ def add_backend_cleanup_task(app):
     """The backend cleanup task can be used to clean up the default result
     backend.
 
-    This task is also added do the periodic task schedule so that it is
-    run every day at midnight, but :program:`celerybeat` must be running
-    for this to be effective.
-
-    Note that not all backends do anything for this, what needs to be
-    done at cleanup is up to each backend, and some backends
-    may even clean up in realtime so that a periodic cleanup is not necessary.
+    If the configured backend requires periodic cleanup this task is also
+    automatically configured to run every day at midnight (requires
+    :program:`celery beat` to be running).
 
     """
-
-    @app.task(name='celery.backend_cleanup', _force_evaluate=True)
+    @app.task(name='celery.backend_cleanup',
+              shared=False, _force_evaluate=True)
     def backend_cleanup():
         app.backend.cleanup()
     return backend_cleanup
@@ -60,58 +59,62 @@ def add_backend_cleanup_task(app):
 
 @shared_task
 def add_unlock_chord_task(app):
-    """The unlock chord task is used by result backends that doesn't
-    have native chord support.
+    """This task is used by result backends without native chord support.
 
-    It creates a task chain polling the header for completion.
+    It joins chords by creating a task chain polling the header for completion.
 
     """
-    from celery.canvas import subtask
+    from celery.canvas import signature
     from celery.exceptions import ChordError
-    from celery.result import from_serializable
+    from celery.result import result_from_tuple
 
     default_propagate = app.conf.CELERY_CHORD_PROPAGATES
 
-    @app.task(name='celery.chord_unlock', max_retries=None,
+    @app.task(name='celery.chord_unlock', max_retries=None, shared=False,
               default_retry_delay=1, ignore_result=True, _force_evaluate=True)
     def unlock_chord(group_id, callback, interval=None, propagate=None,
                      max_retries=None, result=None,
                      Result=app.AsyncResult, GroupResult=app.GroupResult,
-                     from_serializable=from_serializable):
+                     result_from_tuple=result_from_tuple):
         # if propagate is disabled exceptions raised by chord tasks
         # will be sent as part of the result list to the chord callback.
         # Since 3.1 propagate will be enabled by default, and instead
         # the chord callback changes state to FAILURE with the
         # exception set to ChordError.
         propagate = default_propagate if propagate is None else propagate
+        if interval is None:
+            interval = unlock_chord.default_retry_delay
 
         # check if the task group is ready, and if so apply the callback.
         deps = GroupResult(
             group_id,
-            [from_serializable(r, app=app) for r in result],
+            [result_from_tuple(r, app=app) for r in result],
         )
         j = deps.join_native if deps.supports_native_join else deps.join
 
         if deps.ready():
-            callback = subtask(callback)
+            callback = signature(callback, app=app)
             try:
                 ret = j(propagate=propagate)
-            except Exception, exc:
+            except Exception as exc:
                 try:
-                    culprit = deps._failed_join_report().next()
-                    reason = 'Dependency %s raised %r' % (culprit.id, exc)
+                    culprit = next(deps._failed_join_report())
+                    reason = 'Dependency {0.id} raised {1!r}'.format(
+                        culprit, exc,
+                    )
                 except StopIteration:
                     reason = repr(exc)
+
                 app._tasks[callback.task].backend.fail_from_current_stack(
                     callback.id, exc=ChordError(reason),
                 )
             else:
                 try:
                     callback.delay(ret)
-                except Exception, exc:
+                except Exception as exc:
                     app._tasks[callback.task].backend.fail_from_current_stack(
                         callback.id,
-                        exc=ChordError('Callback error: %r' % (exc, )),
+                        exc=ChordError('Callback error: {0!r}'.format(exc)),
                     )
         else:
             return unlock_chord.retry(countdown=interval,
@@ -121,23 +124,23 @@ def add_unlock_chord_task(app):
 
 @shared_task
 def add_map_task(app):
-    from celery.canvas import subtask
+    from celery.canvas import signature
 
-    @app.task(name='celery.map', _force_evaluate=True)
+    @app.task(name='celery.map', shared=False, _force_evaluate=True)
     def xmap(task, it):
-        task = subtask(task).type
-        return [task(value) for value in it]
+        task = signature(task, app=app).type
+        return [task(item) for item in it]
     return xmap
 
 
 @shared_task
 def add_starmap_task(app):
-    from celery.canvas import subtask
+    from celery.canvas import signature
 
-    @app.task(name='celery.starmap', _force_evaluate=True)
+    @app.task(name='celery.starmap', shared=False, _force_evaluate=True)
     def xstarmap(task, it):
-        task = subtask(task).type
-        return [task(*args) for args in it]
+        task = signature(task, app=app).type
+        return [task(*item) for item in it]
     return xstarmap
 
 
@@ -145,7 +148,7 @@ def add_starmap_task(app):
 def add_chunk_task(app):
     from celery.canvas import chunks as _chunks
 
-    @app.task(name='celery.chunks', _force_evaluate=True)
+    @app.task(name='celery.chunks', shared=False, _force_evaluate=True)
     def chunks(task, it, n):
         return _chunks.apply_chunks(task, it, n)
     return chunks
@@ -154,19 +157,20 @@ def add_chunk_task(app):
 @shared_task
 def add_group_task(app):
     _app = app
-    from celery.canvas import maybe_subtask, subtask
-    from celery.result import from_serializable
+    from celery.canvas import maybe_signature, signature
+    from celery.result import result_from_tuple
 
     class Group(app.Task):
         app = _app
         name = 'celery.group'
         accept_magic_kwargs = False
+        _decorated = True
 
         def run(self, tasks, result, group_id, partial_args):
             app = self.app
-            result = from_serializable(result, app)
+            result = result_from_tuple(result, app)
             # any partial args are added to all tasks in the group
-            taskit = (subtask(task).clone(partial_args)
+            taskit = (signature(task, app=app).clone(partial_args)
                       for i, task in enumerate(tasks))
             if self.request.is_eager or app.conf.CELERY_ALWAYS_EAGER:
                 return app.GroupResult(
@@ -178,30 +182,25 @@ def add_group_task(app):
                                    add_to_parent=False) for stask in taskit]
             parent = get_current_worker_task()
             if parent:
-                parent.request.children.append(result)
+                parent.add_trail(result)
             return result
 
         def prepare(self, options, tasks, args, **kwargs):
-            AsyncResult = self.AsyncResult
             options['group_id'] = group_id = (
                 options.setdefault('task_id', uuid()))
 
             def prepare_member(task):
-                task = maybe_subtask(task)
-                opts = task.options
-                opts['group_id'] = group_id
-                try:
-                    tid = opts['task_id']
-                except KeyError:
-                    tid = opts['task_id'] = uuid()
-                return task, AsyncResult(tid)
+                task = maybe_signature(task, app=self.app)
+                task.options['group_id'] = group_id
+                return task, task.freeze()
 
             try:
-                tasks, results = zip(*[prepare_member(task) for task in tasks])
+                tasks, res = list(zip(
+                    *[prepare_member(task) for task in tasks]
+                ))
             except ValueError:  # tasks empty
-                tasks, results = [], []
-            return (tasks, self.app.GroupResult(group_id, results),
-                    group_id, args)
+                tasks, res = [], []
+            return (tasks, self.app.GroupResult(group_id, res), group_id, args)
 
         def apply_async(self, partial_args=(), kwargs={}, **options):
             if self.app.conf.CELERY_ALWAYS_EAGER:
@@ -210,7 +209,7 @@ def add_group_task(app):
                 options, args=partial_args, **kwargs
             )
             super(Group, self).apply_async((
-                list(tasks), result.serializable(), gid, args), **options
+                list(tasks), result.as_tuple(), gid, args), **options
             )
             return result
 
@@ -223,50 +222,55 @@ def add_group_task(app):
 
 @shared_task
 def add_chain_task(app):
-    from celery.canvas import Signature, chord, group, maybe_subtask
+    from celery.canvas import Signature, chord, group, maybe_signature
     _app = app
 
     class Chain(app.Task):
         app = _app
         name = 'celery.chain'
         accept_magic_kwargs = False
+        _decorated = True
 
         def prepare_steps(self, args, tasks):
+            app = self.app
             steps = deque(tasks)
             next_step = prev_task = prev_res = None
             tasks, results = [], []
             i = 0
             while steps:
                 # First task get partial args from chain.
-                task = maybe_subtask(steps.popleft())
+                task = maybe_signature(steps.popleft(), app=app)
                 task = task.clone() if i else task.clone(args)
-                res = task._freeze()
+                res = task.freeze()
                 i += 1
 
-                if isinstance(task, group):
+                if isinstance(task, group) and steps and \
+                        not isinstance(steps[0], group):
                     # automatically upgrade group(..) | s to chord(group, s)
                     try:
                         next_step = steps.popleft()
                         # for chords we freeze by pretending it's a normal
                         # task instead of a group.
-                        res = Signature._freeze(task)
+                        res = Signature.freeze(next_step)
                         task = chord(task, body=next_step, task_id=res.task_id)
                     except IndexError:
-                        pass
+                        pass  # no callback, so keep as group
                 if prev_task:
                     # link previous task to this task.
                     prev_task.link(task)
                     # set the results parent attribute.
-                    res.parent = prev_res
+                    if not res.parent:
+                        res.parent = prev_res
 
-                results.append(res)
-                tasks.append(task)
+                if not isinstance(prev_task, chord):
+                    results.append(res)
+                    tasks.append(task)
                 prev_task, prev_res = task, res
 
             return tasks, results
 
         def apply_async(self, args=(), kwargs={}, group_id=None, chord=None,
-                        task_id=None, **options):
+                        task_id=None, link=None, link_error=None, **options):
             if self.app.conf.CELERY_ALWAYS_EAGER:
                 return self.apply(args, kwargs, **options)
             options.pop('publisher', None)
@@ -279,13 +283,24 @@ def add_chain_task(app):
             if task_id:
                 tasks[-1].set(task_id=task_id)
                 result = tasks[-1].type.AsyncResult(task_id)
+            # make sure we can do a link() and link_error() on a chain object.
+            if link:
+                tasks[-1].set(link=link)
+            # and if any task in the chain fails, call the errbacks
+            if link_error:
+                for task in tasks:
+                    task.set(link_error=link_error)
             tasks[0].apply_async()
             return result
 
-        def apply(self, args=(), kwargs={}, subtask=maybe_subtask, **options):
+        def apply(self, args=(), kwargs={}, signature=maybe_signature,
+                  **options):
+            app = self.app
             last, fargs = None, args  # fargs passed to first task only
             for task in kwargs['tasks']:
-                res = subtask(task).clone(fargs).apply(last and (last.get(), ))
+                res = signature(task, app=app).clone(fargs).apply(
+                    last and (last.get(), ),
+                )
                 res.parent, last, fargs = last, res, None
             return last
     return Chain
@@ -294,10 +309,10 @@ def add_chain_task(app):
 @shared_task
 def add_chord_task(app):
     """Every chord is executed in a dedicated task, so that the chord
-    can be used as a subtask, and this generates the task
+    can be used as a signature, and this generates the task
     responsible for that."""
     from celery import group
-    from celery.canvas import maybe_subtask
+    from celery.canvas import maybe_signature
     _app = app
     default_propagate = app.conf.CELERY_CHORD_PROPAGATES
 
@@ -306,18 +321,22 @@ def add_chord_task(app):
         name = 'celery.chord'
         accept_magic_kwargs = False
         ignore_result = False
+        _decorated = True
 
         def run(self, header, body, partial_args=(), interval=None,
                 countdown=1, max_retries=None, propagate=None,
                 eager=False, **kwargs):
+            app = self.app
             propagate = default_propagate if propagate is None else propagate
             group_id = uuid()
-            AsyncResult = self.app.AsyncResult
+            AsyncResult = app.AsyncResult
             prepare_member = self._prepare_member
 
             # - convert back to group if serialized
             tasks = header.tasks if isinstance(header, group) else header
-            header = group([maybe_subtask(s).clone() for s in tasks])
+            header = group([
+                maybe_signature(s, app=app).clone() for s in tasks
+            ])
             # - eager applies the group inline
             if eager:
                 return header.apply(args=partial_args, task_id=group_id)
@@ -333,8 +352,9 @@ def add_chord_task(app):
                                        propagate=propagate,
                                        result=results)
             # - call the header group, returning the GroupResult.
-            # XXX Python 2.5 doesn't allow kwargs after star-args.
-            return header(*partial_args, **{'task_id': group_id})
+            final_res = header(*partial_args, task_id=group_id)
+
+            return final_res
 
         def _prepare_member(self, task, body, group_id):
             opts = task.options
@@ -346,23 +366,25 @@ def add_chord_task(app):
             opts.update(chord=body, group_id=group_id)
             return task_id
 
-        def apply_async(self, args=(), kwargs={}, task_id=None, **options):
-            if self.app.conf.CELERY_ALWAYS_EAGER:
+        def apply_async(self, args=(), kwargs={}, task_id=None,
+                        group_id=None, chord=None, **options):
+            app = self.app
+            if app.conf.CELERY_ALWAYS_EAGER:
                 return self.apply(args, kwargs, **options)
-            group_id = options.pop('group_id', None)
-            chord = options.pop('chord', None)
             header = kwargs.pop('header')
             body = kwargs.pop('body')
-            header, body = (list(maybe_subtask(header)),
-                            maybe_subtask(body))
-            if group_id:
-                body.set(group_id=group_id)
-            if chord:
-                body.set(chord=chord)
-            callback_id = body.options.setdefault('task_id', task_id or uuid())
+            header, body = (list(maybe_signature(header, app=app)),
+                            maybe_signature(body, app=app))
+            # forward certain options to body
+            if chord is not None:
+                body.options['chord'] = chord
+            if group_id is not None:
+                body.options['group_id'] = group_id
+            [body.link(s) for s in options.pop('link', [])]
+            [body.link_error(s) for s in options.pop('link_error', [])]
+            body_result = body.freeze(task_id)
             parent = super(Chord, self).apply_async((header, body, args),
                                                     kwargs, **options)
-            body_result = self.AsyncResult(callback_id)
             body_result.parent = parent
             return body_result
 
@@ -370,6 +392,6 @@ def add_chord_task(app):
             body = kwargs['body']
             res = super(Chord, self).apply(args, dict(kwargs, eager=True),
                                            **options)
-            return maybe_subtask(body).apply(
+            return maybe_signature(body, app=self.app).apply(
                 args=(res.get(propagate=propagate).get(), ))
     return Chord
