@@ -130,7 +130,7 @@ The :program:`celery worker` command (previously known as ``celeryd``)
     Don't do execv after multiprocessing child fork.
 
 """
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals
 
 import sys
 
