@@ -17,67 +17,121 @@
 
 """
 from __future__ import absolute_import
-from __future__ import with_statement
 
-import heapq
 import threading
 
+from datetime import datetime
+from heapq import heappush, heappop
+from itertools import islice
 from time import time
 
+from kombu.clocks import timetuple
 from kombu.utils import kwdict
 
 from celery import states
-from celery.datastructures import AttributeDict, LRUCache
+from celery.datastructures import AttributeDict
+from celery.five import items, values
+from celery.utils.functional import LRUCache
+from celery.utils.log import get_logger
 
 # The window (in percentage) is added to the workers heartbeat
 # frequency.  If the time between updates exceeds this window,
 # then the worker is considered to be offline.
 HEARTBEAT_EXPIRE_WINDOW = 200
 
+# Max drift between event timestamp and time of event received
+# before we alert that clocks may be unsynchronized.
+HEARTBEAT_DRIFT_MAX = 16
+
+DRIFT_WARNING = """\
+Substantial drift from %s may mean clocks are out of sync.  Current drift is
+%s seconds.  [orig: %s recv: %s]
+"""
+
+logger = get_logger(__name__)
+warn = logger.warning
+
+R_STATE = '<State: events={0.event_count} tasks={0.task_count}>'
+R_WORKER = '<Worker: {0.hostname} ({0.status_string})'
+R_TASK = '<Task: {0.name}({0.uuid}) {0.state}>'
+
+__all__ = ['Worker', 'Task', 'State', 'heartbeat_expires']
+
 
 def heartbeat_expires(timestamp, freq=60,
                       expire_window=HEARTBEAT_EXPIRE_WINDOW):
     return timestamp + freq * (expire_window / 1e2)
 
 
-class Element(AttributeDict):
-    """Base class for worker state elements."""
+def with_unique_field(attr):
+
+    def _decorate_cls(cls):
+
+        def __eq__(this, other):
+            if isinstance(other, this.__class__):
+                return getattr(this, attr) == getattr(other, attr)
+            return NotImplemented
+        cls.__eq__ = __eq__
+
+        def __ne__(this, other):
+            return not this.__eq__(other)
+        cls.__ne__ = __ne__
+
+        def __hash__(this):
+            return hash(getattr(this, attr))
+        cls.__hash__ = __hash__
+
+        return cls
+    return _decorate_cls
 
 
-class Worker(Element):
+@with_unique_field('hostname')
+class Worker(AttributeDict):
     """Worker State."""
     heartbeat_max = 4
     expire_window = HEARTBEAT_EXPIRE_WINDOW
+    pid = None
+    _defaults = {'hostname': None, 'pid': None, 'freq': 60}
 
     def __init__(self, **fields):
-        fields.setdefault('freq', 60)
-        super(Worker, self).__init__(**fields)
+        dict.__init__(self, self._defaults, **fields)
         self.heartbeats = []
 
-    def on_online(self, timestamp=None, **kwargs):
+    def on_online(self, timestamp=None, local_received=None, **kwargs):
         """Callback for the :event:`worker-online` event."""
         self.update(**kwargs)
-        self._heartpush(timestamp)
+        self.update_heartbeat(local_received, timestamp)
 
     def on_offline(self, **kwargs):
         """Callback for the :event:`worker-offline` event."""
         self.update(**kwargs)
         self.heartbeats = []
 
-    def on_heartbeat(self, timestamp=None, **kwargs):
+    def on_heartbeat(self, timestamp=None, local_received=None, **kwargs):
         """Callback for the :event:`worker-heartbeat` event."""
         self.update(**kwargs)
-        self._heartpush(timestamp)
+        self.update_heartbeat(local_received, timestamp)
 
-    def _heartpush(self, timestamp):
-        if timestamp:
-            heapq.heappush(self.heartbeats, timestamp)
-            if len(self.heartbeats) > self.heartbeat_max:
-                self.heartbeats = self.heartbeats[self.heartbeat_max:]
+    def update_heartbeat(self, received, timestamp):
+        if not received or not timestamp:
+            return
+        drift = abs(int(received) - int(timestamp))
+        if drift > HEARTBEAT_DRIFT_MAX:
+            warn(DRIFT_WARNING, self.hostname, drift,
+                 datetime.fromtimestamp(received),
+                 datetime.fromtimestamp(timestamp))
+        heartbeats, hbmax = self.heartbeats, self.heartbeat_max
+        if not heartbeats or (received and received > heartbeats[-1]):
+            heappush(heartbeats, received)
+            if len(heartbeats) > hbmax:
+                heartbeats[:] = heartbeats[hbmax:]
 
     def __repr__(self):
-        return '<Worker: %s (%s)' % (self.hostname,
-                                     self.alive and 'ONLINE' or 'OFFLINE')
+        return R_WORKER.format(self)
+
+    @property
+    def status_string(self):
+        return 'ONLINE' if self.alive else 'OFFLINE'
 
     @property
     def heartbeat_expires(self):
@@ -86,10 +140,15 @@ class Worker(Element):
 
     @property
     def alive(self):
-        return (self.heartbeats and time() < self.heartbeat_expires)
+        return bool(self.heartbeats and time() < self.heartbeat_expires)
+
+    @property
+    def id(self):
+        return '{0.hostname}.{0.pid}'.format(self)
 
 
-class Task(Element):
+@with_unique_field('uuid')
+class Task(AttributeDict):
     """Task State."""
 
     #: How to merge out of order events.
@@ -115,12 +174,13 @@ class Task(Element):
                      revoked=False, args=None, kwargs=None, eta=None,
                      expires=None, retries=None, worker=None, result=None,
                      exception=None, timestamp=None, runtime=None,
-                     traceback=None, exchange=None, routing_key=None)
+                     traceback=None, exchange=None, routing_key=None,
+                     clock=0)
 
     def __init__(self, **fields):
-        super(Task, self).__init__(**dict(self._defaults, **fields))
+        dict.__init__(self, self._defaults, **fields)
 
-    def update(self, state, timestamp, fields):
+    def update(self, state, timestamp, fields, _state=states.state):
         """Update state from new event.
 
         :param state: State from event.
@@ -128,10 +188,11 @@ class Task(Element):
         :param fields: Event data.
 
         """
-        if self.worker:
-            self.worker.on_heartbeat(timestamp=timestamp)
+        time_received = fields.get('local_received') or 0
+        if self.worker and time_received:
+            self.worker.update_heartbeat(time_received, timestamp)
         if state != states.RETRY and self.state != states.RETRY and \
-                states.state(state) < states.state(self.state):
+                _state(state) < _state(self.state):
             # this state logically happens-before the current state, so merge.
             self.merge(state, timestamp, fields)
         else:
@@ -181,8 +242,8 @@ class Task(Element):
         self.revoked = timestamp
         self.update(states.REVOKED, timestamp, fields)
 
-    def on_unknown_event(self, type, timestamp=None, **fields):
-        self.update(type.upper(), timestamp, fields)
+    def on_unknown_event(self, shortype, timestamp=None, **fields):
+        self.update(shortype.upper(), timestamp, fields)
 
     def info(self, fields=None, extra=[]):
         """Information about this task suitable for on-screen display."""
@@ -197,7 +258,7 @@ class Task(Element):
         return dict(_keys())
 
     def __repr__(self):
-        return '<Task: %s(%s) %s>' % (self.name, self.uuid, self.state)
+        return R_TASK.format(self)
 
     @property
     def ready(self):
@@ -217,7 +278,7 @@ class State(object):
                         if workers is None else workers)
         self.tasks = (LRUCache(max_tasks_in_memory)
                       if tasks is None else tasks)
-        self._taskheap = None   # reserved for __reduce__ in 3.1
+        self._taskheap = [] if taskheap is None else taskheap
         self.max_workers_in_memory = max_workers_in_memory
         self.max_tasks_in_memory = max_tasks_in_memory
         self._mutex = threading.Lock()
@@ -247,6 +308,7 @@ class State(object):
             self.tasks.update(in_progress)
         else:
             self.tasks.clear()
+        self._taskheap[:] = []
 
     def _clear(self, ready=True):
         self.workers.clear()
@@ -259,38 +321,56 @@ class State(object):
             return self._clear(ready)
 
     def get_or_create_worker(self, hostname, **kwargs):
-        """Get or create worker by hostname."""
+        """Get or create worker by hostname.
+
+        Return tuple of ``(worker, was_created)``.
+        """
         try:
             worker = self.workers[hostname]
             worker.update(kwargs)
+            return worker, False
         except KeyError:
             worker = self.workers[hostname] = Worker(
                 hostname=hostname, **kwargs)
-        return worker
+            return worker, True
 
     def get_or_create_task(self, uuid):
         """Get or create task by uuid."""
         try:
-            return self.tasks[uuid]
+            return self.tasks[uuid], False
         except KeyError:
             task = self.tasks[uuid] = Task(uuid=uuid)
-            return task
+            return task, True
 
     def worker_event(self, type, fields):
         """Process worker event."""
-        hostname = fields.pop('hostname', None)
-        if hostname:
-            worker = self.get_or_create_worker(hostname)
-            handler = getattr(worker, 'on_%s' % type, None)
+        try:
+            hostname = fields['hostname']
+        except KeyError:
+            pass
+        else:
+            worker, created = self.get_or_create_worker(hostname)
+            handler = getattr(worker, 'on_' + type, None)
             if handler:
                 handler(**fields)
+            return worker, created
 
-    def task_event(self, type, fields):
+    def task_event(self, type, fields, timetuple=timetuple):
         """Process task event."""
         uuid = fields['uuid']
         hostname = fields['hostname']
-        worker = self.get_or_create_worker(hostname)
-        task = self.get_or_create_task(uuid)
+        worker, _ = self.get_or_create_worker(hostname)
+        task, created = self.get_or_create_task(uuid)
+        task.worker = worker
+        maxtasks = self.max_tasks_in_memory * 2
+
+        taskheap = self._taskheap
+        timestamp = fields.get('timestamp') or 0
+        clock = 0 if type == 'sent' else fields.get('clock')
+        heappush(taskheap, timetuple(clock, timestamp, worker.id, task))
+        if len(taskheap) > maxtasks:
+            heappop(taskheap)
+
         handler = getattr(task, 'on_' + type, None)
         if type == 'received':
             self.task_count += 1
@@ -298,7 +378,7 @@ class State(object):
             handler(**fields)
         else:
             task.on_unknown_event(type, **fields)
-        task.worker = worker
+        return created
 
     def event(self, event):
         with self._mutex:
@@ -316,62 +396,57 @@ class State(object):
             self.event_callback(self, event)
 
     def itertasks(self, limit=None):
-        for index, row in enumerate(self.tasks.iteritems()):
+        for index, row in enumerate(items(self.tasks)):
             yield row
             if limit and index + 1 >= limit:
                 break
 
-    def tasks_by_timestamp(self, limit=None):
-        """Get tasks by timestamp.
-
-        Returns a list of `(uuid, task)` tuples.
-
-        """
-        return self._sort_tasks_by_time(self.itertasks(limit))
-
-    def _sort_tasks_by_time(self, tasks):
-        """Sort task items by time."""
-        return sorted(tasks, key=lambda t: t[1].timestamp,
-                      reverse=True)
+    def tasks_by_time(self, limit=None):
+        """Generator giving tasks ordered by time,
+        in ``(uuid, Task)`` tuples."""
+        seen = set()
+        for evtup in islice(reversed(self._taskheap), 0, limit):
+            uuid = evtup[3].uuid
+            if uuid not in seen:
+                yield uuid, evtup[3]
+                seen.add(uuid)
+    tasks_by_timestamp = tasks_by_time
 
     def tasks_by_type(self, name, limit=None):
         """Get all tasks by type.
 
-        Returns a list of `(uuid, task)` tuples.
+        Return a list of ``(uuid, Task)`` tuples.
 
         """
-        sorted_tasks = self._sort_tasks_by_time(
-            (uuid, task) for uuid, task in self.tasks.iteritems()
-            if task.name == name)
-
-        return sorted_tasks[0:limit or None]
+        return islice(
+            ((uuid, task) for uuid, task in self.tasks_by_time()
+             if task.name == name),
+            0, limit,
+        )
 
     def tasks_by_worker(self, hostname, limit=None):
         """Get all tasks by worker.
 
-        Returns a list of `(uuid, task)` tuples.
-
         """
-        return self._sort_tasks_by_time(
-            (uuid, task) for uuid, task in self.itertasks(limit)
-            if task.worker.hostname == hostname)
+        return islice(
+            ((uuid, task) for uuid, task in self.tasks_by_time()
+             if task.worker.hostname == hostname),
+            0, limit,
+        )
 
     def task_types(self):
-        """Returns a list of all seen task types."""
-        return list(sorted(set(task.name for task in self.tasks.itervalues())))
+        """Return a list of all seen task types."""
+        return list(sorted(set(task.name for task in values(self.tasks))))
 
     def alive_workers(self):
-        """Returns a list of (seemingly) alive workers."""
-        return [w for w in self.workers.values() if w.alive]
+        """Return a list of (seemingly) alive workers."""
+        return [w for w in values(self.workers) if w.alive]
 
     def __repr__(self):
-        return '<ClusterState: events=%s tasks=%s>' % (self.event_count,
-                                                       self.task_count)
+        return R_STATE.format(self)
 
     def __reduce__(self):
         return self.__class__, (
-            self.event_callback, self.workers, self.tasks, None,
+            self.event_callback, self.workers, self.tasks, self._taskheap,
             self.max_workers_in_memory, self.max_tasks_in_memory,
         )
-
-state = State()
