@@ -10,23 +10,46 @@ from __future__ import absolute_import
 
 import anyjson
 import sys
-import urllib2
 
-from urllib import urlencode
-from urlparse import urlparse
 try:
-    from urlparse import parse_qsl
+    from urllib.parse import parse_qsl, urlencode, urlparse   # Py3
 except ImportError:  # pragma: no cover
-    from cgi import parse_qsl  # noqa
+    from urllib import urlencode              # noqa
+    from urlparse import urlparse, parse_qsl  # noqa
 
-from celery import __version__ as celery_version
+from celery import shared_task, __version__ as celery_version
+from celery.five import items, reraise
 from celery.utils.log import get_task_logger
-from .base import Task as BaseTask
+
+__all__ = ['InvalidResponseError', 'RemoteExecuteError', 'UnknownStatusError',
+           'HttpDispatch', 'dispatch', 'URL']
 
 GET_METHODS = frozenset(['GET', 'HEAD'])
 logger = get_task_logger(__name__)
 
 
+if sys.version_info[0] == 3:  # pragma: no cover
+
+    from urllib.request import Request, urlopen
+
+    def utf8dict(tup):
+        if not isinstance(tup, dict):
+            return dict(tup)
+        return tup
+
+else:
+
+    from urllib2 import Request, urlopen  # noqa
+
+    def utf8dict(tup):  # noqa
+        """With a dict's items() tuple return a new dict with any utf-8
+        keys/values encoded."""
+        return dict(
+            (k.encode('utf-8'),
+             v.encode('utf-8') if isinstance(v, unicode) else v)
+            for k, v in tup)
+
+
 class InvalidResponseError(Exception):
     """The remote server gave an invalid response."""
 
@@ -39,37 +62,15 @@ class UnknownStatusError(InvalidResponseError):
     """The remote server gave an unknown status."""
 
 
-def maybe_utf8(value):
-    """Encode to utf-8, only if the value is Unicode."""
-    if isinstance(value, unicode):
-        return value.encode('utf-8')
-    return value
-
-
-if sys.version_info[0] == 3:  # pragma: no cover
-
-    def utf8dict(tup):
-        if not isinstance(tup, dict):
-            return dict(tup)
-        return tup
-else:
-
-    def utf8dict(tup):  # noqa
-        """With a dict's items() tuple return a new dict with any utf-8
-        keys/values encoded."""
-        return dict((key.encode('utf-8'), maybe_utf8(value))
-                    for key, value in tup)
-
-
 def extract_response(raw_response, loads=anyjson.loads):
     """Extract the response text from a raw JSON response."""
     if not raw_response:
         raise InvalidResponseError('Empty response')
     try:
         payload = loads(raw_response)
-    except ValueError, exc:
-        raise InvalidResponseError, InvalidResponseError(
-            str(exc)), sys.exc_info()[2]
+    except ValueError as exc:
+        reraise(InvalidResponseError, InvalidResponseError(
+            str(exc)), sys.exc_info()[2])
 
     status = payload['status']
     if status == 'success':
@@ -106,15 +107,15 @@ class MutableURL(object):
 
     def __str__(self):
         scheme, netloc, path, params, query, fragment = self.parts
-        query = urlencode(utf8dict(self.query.items()))
+        query = urlencode(utf8dict(items(self.query)))
         components = [scheme + '://', netloc, path or '/',
-                      ';%s' % params if params else '',
-                      '?%s' % query if query else '',
-                      '#%s' % fragment if fragment else '']
+                      ';{0}'.format(params) if params else '',
+                      '?{0}'.format(query) if query else '',
+                      '#{0}'.format(fragment) if fragment else '']
         return ''.join(c for c in components if c)
 
     def __repr__(self):
-        return '<%s: %s>' % (self.__class__.__name__, str(self))
+        return '<{0}: {1}>'.format(type(self).__name__, self)
 
 
 class HttpDispatch(object):
@@ -127,21 +128,21 @@ class HttpDispatch(object):
     :param logger: Logger used for user/system feedback.
 
     """
-    user_agent = 'celery/%s' % celery_version
+    user_agent = 'celery/{version}'.format(version=celery_version)
     timeout = 5
 
     def __init__(self, url, method, task_kwargs, **kwargs):
         self.url = url
         self.method = method
         self.task_kwargs = task_kwargs
-        self.logger = kwargs.get("logger") or logger
+        self.logger = kwargs.get('logger') or logger
 
     def make_request(self, url, method, params):
-        """Makes an HTTP request and returns the response."""
-        request = urllib2.Request(url, params)
-        for key, val in self.http_headers.items():
+        """Perform HTTP request and return the response."""
+        request = Request(url, params)
+        for key, val in items(self.http_headers):
             request.add_header(key, val)
-        response = urllib2.urlopen(request)         # user catches errors.
+        response = urlopen(request)  # user catches errors.
         return response.read()
 
     def dispatch(self):
@@ -151,7 +152,7 @@ class HttpDispatch(object):
         if self.method in GET_METHODS:
             url.query.update(self.task_kwargs)
         else:
-            params = urlencode(utf8dict(self.task_kwargs.items()))
+            params = urlencode(utf8dict(items(self.task_kwargs)))
         raw_response = self.make_request(str(url), self.method, params)
         return extract_response(raw_response)
 
@@ -161,7 +162,9 @@ class HttpDispatch(object):
         return headers
 
 
-class HttpDispatchTask(BaseTask):
+@shared_task(name='celery.http_dispatch', bind=True,
+             url=None, method=None, accept_magic_kwargs=False)
+def dispatch(self, url=None, method='GET', **kwargs):
     """Task dispatching to an URL.
 
     :keyword url: The URL location of the HTTP callback task.
@@ -182,15 +185,9 @@ class HttpDispatchTask(BaseTask):
         argument, as this attribute is intended for subclasses.
 
     """
-
-    url = None
-    method = None
-    accept_magic_kwargs = False
-
-    def run(self, url=None, method='GET', **kwargs):
-        url = url or self.url
-        method = method or self.method
-        return HttpDispatch(url, method, kwargs).dispatch()
+    return HttpDispatch(
+        url or self.url, method or self.method, kwargs,
+    ).dispatch()
 
 
 class URL(MutableURL):
@@ -200,14 +197,21 @@ class URL(MutableURL):
 
     :param url: URL to request.
     :keyword dispatcher: Class used to dispatch the request.
-        By default this is :class:`HttpDispatchTask`.
+        By default this is :func:`dispatch`.
 
     """
-    dispatcher = HttpDispatchTask
+    dispatcher = None
 
-    def __init__(self, url, dispatcher=None):
+    def __init__(self, url, dispatcher=None, app=None):
         super(URL, self).__init__(url)
+        self.app = app
         self.dispatcher = dispatcher or self.dispatcher
+        if self.dispatcher is None:
+            # Get default dispatcher
+            self.dispatcher = (
+                self.app.tasks['celery.http_dispatch'] if self.app
+                else dispatch
+            )
 
     def get_async(self, **kwargs):
         return self.dispatcher.delay(str(self), 'GET', **kwargs)
