@@ -1,5 +1,6 @@
 from __future__ import absolute_import
 
+import gc
 import os
 import itertools
 
@@ -426,6 +427,8 @@ class test_App(AppCase):
         app1.close()
         del(app1)
 
+        gc.collect()
+
         # weakref removed from list when app goes out of scope.
         with self.assertRaises(StopIteration):
             next(app for app in _state._get_active_apps() if id(app) == appid)
