@@ -7,7 +7,6 @@ from collections import defaultdict
 from datetime import datetime, timedelta
 
 from kombu import pidbox
-from mock import Mock, patch, call
 
 from celery.datastructures import AttributeDict
 from celery.five import Queue as FastQueue
@@ -22,7 +21,7 @@ from celery.worker.state import revoked
 from celery.worker.control import Panel
 from celery.worker.pidbox import Pidbox, gPidbox
 
-from celery.tests.case import AppCase
+from celery.tests.case import AppCase, Mock, call, patch
 
 hostname = socket.gethostname()
 
