@@ -1,9 +1,15 @@
 from __future__ import absolute_import
 
+import os
+
 from celery import Celery
+
 from django.conf import settings
 
-app = Celery('proj.celery')
+# set the default Django settings module for the 'celery' program.
+os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'proj.settings')
+
+app = Celery('proj')
 
 # Using a string here means the worker will not have to
 # pickle the object when using Windows.
